use crate::RequestParser;
use mio::{net::TcpStream, Token};
use std::net::SocketAddr;

pub struct Connection<Parser: RequestParser> {
    pub stream: TcpStream,
    pub state: ConnectionState<Parser>,
    pub client_address: SocketAddr,
    pub token: Token,
}

impl<Parser: RequestParser> Connection<Parser> {
    pub fn new(stream: TcpStream, client_address: SocketAddr, token: Token) -> Self {
        Self {
            stream,
            client_address,
            token,
            state: ConnectionState::Incoming(Parser::new(client_address)),
        }
    }
}

pub enum ConnectionState<Parser: RequestParser> {
    /// The request has not yet been fully received.
    Incoming(Parser),
    /// The request is valid and needs to be processed.
    // Processing(Parser::Req),
    /// A response has been generated for the request and is waiting to be sent.
    Outgoing(Parser::Res),
}
